import binascii
import codecs

import logbook

from construct import *

from regipy.exceptions import RegistryKeyNotFoundException
from regipy.hive_types import NTUSER_HIVE_TYPE
from regipy.plugins.plugin import Plugin
from regipy.utils import convert_wintime

logger = logbook.Logger(__name__)

WORD_WHEEL_QUERY_KEY_PATH = r'\Software\Microsoft\Windows\CurrentVersion\Explorer\WordWheelQuery'


class WordWheelQueryPlugin(Plugin):
    NAME = 'word_wheel_query'
    DESCRIPTION = 'Parse the word wheel query artifact'
    COMPATIBLE_HIVE = NTUSER_HIVE_TYPE

    def run(self):
        try:
            subkey = self.registry_hive.get_key(WORD_WHEEL_QUERY_KEY_PATH)
        except RegistryKeyNotFoundException as ex:
            logger.error(f'Could not find {self.NAME} plugin data at: {WORD_WHEEL_QUERY_KEY_PATH}: {ex}')
            return None

        timestamp = convert_wintime(subkey.header.last_modified, as_json=self.as_json)

        mru_list_order = subkey.get_value('MRUListEx')

        # If this is the value, the list is empty
        if mru_list_order == 0xffffffff:
            return None

        for i, entry_name in enumerate(GreedyRange(Int32ul).parse(mru_list_order)):
            entry_value = subkey.get_value(str(entry_name))

            if not entry_value:
                continue

            self.entries.append({
                'last_write': timestamp,
                'mru_id': entry_name,
                'order': i,
                'name': CString('utf-16').parse(entry_value)
            })



import cProfile
import io
import lzma
import os
import pstats
from contextlib import contextmanager
from pathlib import Path
from pstats import SortKey
from tempfile import mktemp

import logbook

from regipy.registry import RegistryHive

logger = logbook.Logger(__name__)


@contextmanager
def profiling():
    pr = cProfile.Profile()
    pr.enable()
    yield
    pr.disable()
    s = io.StringIO()
    sortby = SortKey.CUMULATIVE
    ps = pstats.Stats(pr, stream=s).sort_stats(sortby)
    ps.print_stats()
    print(s.getvalue())


@contextmanager
def get_file_from_tests(file_name):
    path = str(Path(__file__).parent.joinpath('data').joinpath(file_name))
    tempfile_path = mktemp()
    with open(tempfile_path, 'wb') as tmp:
        with lzma.open(path) as f:
            tmp.write(f.read())
    yield tempfile_path
    os.remove(tempfile_path)


registry_path = 'SYSTEM.xz'
print(f'Iterating over all subkeys in {registry_path}')
with profiling():
    with get_file_from_tests(registry_path) as reg:
        registry_hive = RegistryHive(reg)
        keys = [x for x in registry_hive.recurse_subkeys()]
print(f'Done.')





import lzma
import os
from contextlib import contextmanager

from pathlib import Path
from tempfile import mktemp

import pytest


def extract_lzma(path):
    tempfile_path = mktemp()
    with open(tempfile_path, 'wb') as tmp:
        with lzma.open(path) as f:
            tmp.write(f.read())
    return tempfile_path


@pytest.fixture()
def temp_output_file():
    tempfile_path = mktemp()
    yield tempfile_path
    os.remove(tempfile_path)


@pytest.fixture(scope='module')
def test_data_dir():
    return str(Path(__file__).parent.joinpath('data'))


@pytest.fixture(scope='module')
def ntuser_hive(test_data_dir):
    temp_path = extract_lzma(os.path.join(test_data_dir, 'NTUSER.DAT.xz'))
    yield temp_path
    os.remove(temp_path)


@pytest.fixture(scope='module')
def software_hive(test_data_dir):
    temp_path = extract_lzma(os.path.join(test_data_dir, 'SOFTWARE.xz'))
    yield temp_path
    os.remove(temp_path)


@pytest.fixture(scope='module')
def system_hive(test_data_dir):
    temp_path = extract_lzma(os.path.join(test_data_dir, 'SYSTEM.xz'))
    yield temp_path
    os.remove(temp_path)


@pytest.fixture(scope='module')
def amcache_hive(test_data_dir):
    temp_path = extract_lzma(os.path.join(test_data_dir, 'amcache.hve.xz'))
    yield temp_path
    os.remove(temp_path)


@pytest.fixture(scope='module')
def second_hive_path(test_data_dir):
    temp_path = extract_lzma(os.path.join(test_data_dir, 'NTUSER_modified.DAT.xz'))
    yield temp_path
    os.remove(temp_path)


@pytest.fixture(scope='module')
def transaction_ntuser(test_data_dir):
    temp_path = extract_lzma(os.path.join(test_data_dir, 'transactions_NTUSER.DAT.xz'))
    yield temp_path
    os.remove(temp_path)


@pytest.fixture(scope='module')
def transaction_log(test_data_dir):
    temp_path = extract_lzma(os.path.join(test_data_dir, 'transactions_ntuser.dat.log1.xz'))
    yield temp_path
    os.remove(temp_path)


@pytest.fixture(scope='module')
def transaction_system(test_data_dir):
    temp_path = extract_lzma(os.path.join(test_data_dir, 'SYSTEM_B.xz'))
    yield temp_path
    os.remove(temp_path)


@pytest.fixture(scope='module')
def transaction_log_1(test_data_dir):
    temp_path = extract_lzma(os.path.join(test_data_dir, 'SYSTEM_B.LOG1.xz'))
    yield temp_path
    os.remove(temp_path)


@pytest.fixture(scope='module')
def transaction_log_2(test_data_dir):
    temp_path = extract_lzma(os.path.join(test_data_dir, 'SYSTEM_B.LOG2.xz'))
    yield temp_path
    os.remove(temp_path)
